@@ -584,10 +584,8 @@ void ImportDependencies::filter(const ViewerContext &vContext)
 {
     QMap<QString, CoreImport> newCoreImports;
     QMap<ImportKey, QStringList> newImportCache;
-    QMapIterator<QString, CoreImport> j(m_coreImports);
     bool hasChanges = false;
-    while (j.hasNext()) {
-        j.next();
+    for (auto j = m_coreImports.cbegin(), end = m_coreImports.cend(); j != end; ++j) {
         const CoreImport &cImport = j.value();
         if (vContext.languageIsCompatible(cImport.language)) {
             QList<Export> newExports;
@@ -717,8 +715,8 @@ ImportDependencies::ImportElements ImportDependencies::candidateImports(
 QList<DependencyInfo::ConstPtr> ImportDependencies::createDependencyInfos(
         const ImportKey &mainDoc, const ViewerContext &vContext) const
 {
-    Q_UNUSED(mainDoc);
-    Q_UNUSED(vContext);
+    Q_UNUSED(mainDoc)
+    Q_UNUSED(vContext)
     QList<DependencyInfo::ConstPtr> res;
     QTC_CHECK(false);
     return res;
@@ -897,8 +895,8 @@ public:
                     const Export &e,
                     const CoreImport &cI) const
     {
-        Q_UNUSED(m);
-        Q_UNUSED(cI);
+        Q_UNUSED(m)
+        Q_UNUSED(cI)
         imports.insert(e.exportName.flatKey());
         return true;
     }
@@ -923,30 +921,24 @@ QSet<ImportKey> ImportDependencies::subdirImports(
 
 void ImportDependencies::checkConsistency() const
 {
-    QMapIterator<ImportKey, QStringList> j(m_importCache);
-    while (j.hasNext()) {
-        j.next();
-        foreach (const QString &s, j.value()) {
+    for (auto j = m_importCache.cbegin(), end = m_importCache.cend(); j != end; ++j) {
+        for (const QString &s : j.value()) {
             bool found = false;
             foreach (const Export &e, m_coreImports.value(s).possibleExports)
                 if (e.exportName == j.key())
                     found = true;
-            Q_ASSERT(found); Q_UNUSED(found);
+            Q_ASSERT(found); Q_UNUSED(found)
         }
     }
-    QMapIterator<QString,CoreImport> i(m_coreImports);
-    while (i.hasNext()) {
-        i.next();
+    for (auto i = m_coreImports.cbegin(), end = m_coreImports.cend(); i != end; ++i) {
         foreach (const Export &e, i.value().possibleExports) {
             if (!m_importCache.value(e.exportName).contains(i.key())) {
                 qCWarning(importsLog) << e.exportName.toString();
                 qCWarning(importsLog) << i.key();
 
-                QMapIterator<ImportKey, QStringList> j(m_importCache);
-                while (j.hasNext()) {
-                    j.next();
+                for (auto j = m_importCache.cbegin(), end = m_importCache.cend(); j != end; ++j)
                     qCWarning(importsLog) << j.key().toString() << j.value();
-                }
+
                 qCWarning(importsLog) << m_importCache.contains(e.exportName);
                 qCWarning(importsLog) << m_importCache.value(e.exportName);
             }
