@@ -51,115 +51,18 @@ bool QmlItemNode::isItemOrWindow(const ModelNode &modelNode)
     if (modelNode.metaInfo().isSubclassOf("QtQuick.Item"))
         return true;
 
-    if (modelNode.metaInfo().isSubclassOf("QtQuick.Window.Window") && modelNode.isRootNode())
-        return true;
-
-    if (modelNode.metaInfo().isSubclassOf("QtQuick.Controls.Popup"))
+    if (modelNode.metaInfo().isGraphicalItem()  && modelNode.isRootNode())
         return true;
 
     return false;
 }
 
-static QmlItemNode createQmlItemNodeFromSource(AbstractView *view, const QString &source, const QPointF &position)
+QmlItemNode QmlItemNode::createQmlItemNode(AbstractView *view,
+                                           const ItemLibraryEntry &itemLibraryEntry,
+                                           const QPointF &position,
+                                           QmlItemNode parentQmlItemNode)
 {
-    QScopedPointer<Model> inputModel(Model::create("QtQuick.Item", 1, 0, view->model()));
-    inputModel->setFileUrl(view->model()->fileUrl());
-    QPlainTextEdit textEdit;
-
-    textEdit.setPlainText(source);
-    NotIndentingTextEditModifier modifier(&textEdit);
-
-    QScopedPointer<RewriterView> rewriterView(new RewriterView(RewriterView::Amend, 0));
-    rewriterView->setCheckSemanticErrors(false);
-    rewriterView->setTextModifier(&modifier);
-    inputModel->setRewriterView(rewriterView.data());
-
-    if (rewriterView->errors().isEmpty() && rewriterView->rootModelNode().isValid()) {
-        ModelNode rootModelNode = rewriterView->rootModelNode();
-        inputModel->detachView(rewriterView.data());
-
-        rootModelNode.variantProperty("x").setValue(qRound(position.x()));
-        rootModelNode.variantProperty("y").setValue(qRound(position.y()));
-
-        ModelMerger merger(view);
-        return merger.insertModel(rootModelNode);
-    }
-
-    return QmlItemNode();
-}
-
-
-QmlItemNode QmlItemNode::createQmlItemNode(AbstractView *view, const ItemLibraryEntry &itemLibraryEntry, const QPointF &position, QmlItemNode parentQmlItemNode)
-{
-    if (!parentQmlItemNode.isValid())
-        parentQmlItemNode = QmlItemNode(view->rootModelNode());
-
-    Q_ASSERT(parentQmlItemNode.isValid());
-
-    NodeAbstractProperty parentProperty = parentQmlItemNode.defaultNodeAbstractProperty();
-
-    return QmlItemNode::createQmlItemNode(view, itemLibraryEntry, position, parentProperty);
-}
-
-QmlItemNode QmlItemNode::createQmlItemNode(AbstractView *view, const ItemLibraryEntry &itemLibraryEntry, const QPointF &position, NodeAbstractProperty parentproperty)
-{
-    QmlItemNode newQmlItemNode;
-
-    try {
-        RewriterTransaction transaction = view->beginRewriterTransaction(QByteArrayLiteral("QmlItemNode::createQmlItemNode"));
-
-        NodeMetaInfo metaInfo = view->model()->metaInfo(itemLibraryEntry.typeName());
-
-        int minorVersion = metaInfo.minorVersion();
-        int majorVersion = metaInfo.majorVersion();
-
-        typedef QPair<PropertyName, QString> PropertyBindingEntry;
-        QList<PropertyBindingEntry> propertyBindingList;
-        if (itemLibraryEntry.qmlSource().isEmpty()) {
-            QList<QPair<PropertyName, QVariant> > propertyPairList;
-            if (!position.isNull()) {
-                propertyPairList.append({PropertyName("x"), QVariant(qRound(position.x()))});
-                propertyPairList.append({PropertyName("y"), QVariant(qRound(position.y()))});
-            }
-
-            foreach (const PropertyContainer &property, itemLibraryEntry.properties()) {
-                if (property.type() == QStringLiteral("binding")) {
-                    propertyBindingList.append(PropertyBindingEntry(property.name(), property.value().toString()));
-                } else {
-                    propertyPairList.append({property.name(), property.value()});
-                }
-            }
-
-            newQmlItemNode = QmlItemNode(view->createModelNode(itemLibraryEntry.typeName(), majorVersion, minorVersion, propertyPairList));
-        } else {
-            newQmlItemNode = createQmlItemNodeFromSource(view, itemLibraryEntry.qmlSource(), position);
-        }
-
-        if (parentproperty.isValid())
-            parentproperty.reparentHere(newQmlItemNode);
-
-        if (!newQmlItemNode.isValid())
-            return newQmlItemNode;
-
-        newQmlItemNode.setId(view->generateNewId(itemLibraryEntry.name()));
-
-        if (!view->currentState().isBaseState()) {
-            newQmlItemNode.modelNode().variantProperty("opacity").setValue(0);
-            newQmlItemNode.setVariantProperty("opacity", 1);
-        }
-
-        foreach (const PropertyBindingEntry &propertyBindingEntry, propertyBindingList)
-            newQmlItemNode.modelNode().bindingProperty(propertyBindingEntry.first).setExpression(propertyBindingEntry.second);
-
-        Q_ASSERT(newQmlItemNode.isValid());
-    }
-    catch (const RewritingException &e) {
-        e.showException();
-    }
-
-    Q_ASSERT(newQmlItemNode.isValid());
-
-    return newQmlItemNode;
+    return QmlItemNode(createQmlObjectNode(view, itemLibraryEntry, position, parentQmlItemNode));
 }
 
 QmlItemNode QmlItemNode::createQmlItemNodeFromImage(AbstractView *view, const QString &imageName, const QPointF &position, QmlItemNode parentQmlItemNode)
@@ -176,10 +79,8 @@ QmlItemNode QmlItemNode::createQmlItemNodeFromImage(AbstractView *view, const QS
 {
     QmlItemNode newQmlItemNode;
 
-    if (parentproperty.isValid()) {
-        RewriterTransaction transaction = view->beginRewriterTransaction(QByteArrayLiteral("QmlItemNode::createQmlItemNodeFromImage"));
-
-        if (view->model()->hasNodeMetaInfo("QtQuick.Image")) {
+    if (parentproperty.isValid() && view->model()->hasNodeMetaInfo("QtQuick.Image")) {
+        view->executeInTransaction("QmlItemNode::createQmlItemNodeFromImage", [=, &newQmlItemNode, &parentproperty](){
             NodeMetaInfo metaInfo = view->model()->metaInfo("QtQuick.Image");
             QList<QPair<PropertyName, QVariant> > propertyPairList;
             propertyPairList.append({PropertyName("x"), QVariant(qRound(position.x()))});
@@ -199,14 +100,10 @@ QmlItemNode QmlItemNode::createQmlItemNodeFromImage(AbstractView *view, const QS
 
             newQmlItemNode.setId(view->generateNewId(QLatin1String("image")));
 
-            if (!view->currentState().isBaseState()) {
-                newQmlItemNode.modelNode().variantProperty("opacity").setValue(0);
-                newQmlItemNode.setVariantProperty("opacity", 1);
-            }
+            newQmlItemNode.modelNode().variantProperty("fillMode").setEnumeration("Image.PreserveAspectFit");
 
             Q_ASSERT(newQmlItemNode.isValid());
-        }
-        Q_ASSERT(newQmlItemNode.isValid());
+        });
     }
 
     return newQmlItemNode;
@@ -222,39 +119,6 @@ bool QmlItemNode::isValidQmlItemNode(const ModelNode &modelNode)
     return isValidQmlObjectNode(modelNode) && modelNode.metaInfo().isValid() && isItemOrWindow(modelNode);
 }
 
-bool QmlItemNode::isRootNode() const
-{
-    return modelNode().isValid() && modelNode().isRootNode();
-}
-
-QStringList QmlModelStateGroup::names() const
-{
-    QStringList returnList;
-
-    if (!modelNode().isValid())
-        throw new InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
-
-    if (modelNode().property("states").isNodeListProperty()) {
-        foreach (const ModelNode &node, modelNode().nodeListProperty("states").toModelNodeList()) {
-            if (QmlModelState::isValidQmlModelState(node))
-                returnList.append(QmlModelState(node).name());
-        }
-    }
-    return returnList;
-}
-
-/**
-  \brief Returns list of states (without 'base state').
-  The list contains all states defined by this item.
-  */
-QmlModelStateGroup QmlItemNode::states() const
-{
-    if (isValid())
-        return QmlModelStateGroup(modelNode());
-    else
-        return QmlModelStateGroup();
-}
-
 QList<QmlItemNode> QmlItemNode::children() const
 {
     QList<ModelNode> childrenList;
@@ -423,6 +287,11 @@ QRectF  QmlItemNode::instanceBoundingRect() const
     return QRectF(QPointF(0, 0), nodeInstance().size());
 }
 
+QRectF  QmlItemNode::instanceSceneBoundingRect() const
+{
+    return QRectF(instanceScenePosition(), nodeInstance().size());
+}
+
 QRectF QmlItemNode::instancePaintedBoundingRect() const
 {
     return nodeInstance().boundingRect();
@@ -465,7 +334,7 @@ QPointF QmlItemNode::instanceScenePosition() const
      else if (modelNode().hasParentProperty() && QmlItemNode::isValidQmlItemNode(modelNode().parentProperty().parentModelNode()))
         return QmlItemNode(modelNode().parentProperty().parentModelNode()).instanceSceneTransform().map(nodeInstance().position());
 
-    return QPointF();
+    return {};
 }
 
 QPointF QmlItemNode::instancePosition() const
@@ -498,67 +367,11 @@ QPixmap QmlItemNode::instanceBlurredRenderPixmap() const
     return nodeInstance().blurredRenderPixmap();
 }
 
-QList<QmlModelState> QmlModelStateGroup::allStates() const
-{
-    QList<QmlModelState> returnList;
-
-    if (!modelNode().isValid())
-        throw new InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
-
-    if (modelNode().property("states").isNodeListProperty()) {
-        foreach (const ModelNode &node, modelNode().nodeListProperty("states").toModelNodeList()) {
-            if (QmlModelState::isValidQmlModelState(node))
-                returnList.append(node);
-        }
-    }
-    return returnList;
-}
-
-QString QmlItemNode::simplifiedTypeName() const
-{
-    return modelNode().simplifiedTypeName();
-}
-
 uint qHash(const QmlItemNode &node)
 {
     return qHash(node.modelNode());
 }
 
-QmlModelState QmlModelStateGroup::addState(const QString &name)
-{
-    if (!modelNode().isValid())
-        throw new InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
-
-    ModelNode newState = QmlModelState::createQmlState(
-        modelNode().view(), {{PropertyName("name"), QVariant(name)}});
-    modelNode().nodeListProperty("states").reparentHere(newState);
-
-    return newState;
-}
-
-void QmlModelStateGroup::removeState(const QString &name)
-{
-    if (!modelNode().isValid())
-        throw new InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
-
-    if (state(name).isValid())
-        state(name).modelNode().destroy();
-}
-
-QmlModelState QmlModelStateGroup::state(const QString &name) const
-{
-    if (!modelNode().isValid())
-        throw new InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
-
-    if (modelNode().property("states").isNodeListProperty()) {
-        foreach (const ModelNode &node, modelNode().nodeListProperty("states").toModelNodeList()) {
-            if (QmlModelState(node).name() == name)
-                return node;
-        }
-    }
-    return QmlModelState();
-}
-
 QList<ModelNode> toModelNodeList(const QList<QmlItemNode> &qmlItemNodeList)
 {
     QList<ModelNode> modelNodeList;
